package agentkindboxreqvalidator

import (
	"context"
	"errors"
	param "git.gocasts.ir/ebhomengo/niki/param/agent/kind_box_req"
	errmsg "git.gocasts.ir/ebhomengo/niki/pkg/err_msg"
	richerror "git.gocasts.ir/ebhomengo/niki/pkg/rich_error"
	validation "github.com/go-ozzo/ozzo-validation/v4"
)

func (v Validator) ValidateDeliver(ctx context.Context, req param.DeliverKindBoxReqRequest) (map[string]string, error) {
	const op = "adminkindboxreqvalidator.ValidateAssignSender"

	if err := validation.ValidateStruct(&req,
		validation.Field(&req.KindBoxReqID, validation.Required, validation.By(v.doesKindBoxRequestExist(ctx)), validation.By(v.checkKindBoxReqStatusForDelivering(ctx))),
		validation.Field(&req.SerialNumbers, validation.By(v.IsSerialNumbersCountValid(ctx, req.KindBoxReqID)))); err != nil {

		fieldErrors := make(map[string]string)

		var errV validation.Errors
		if errors.As(err, &errV) {
			for key, value := range errV {
				if value != nil {
					fieldErrors[key] = value.Error()
				}
			}
		}

		return fieldErrors, richerror.New(op).
			WithMessage(errmsg.ErrorMsgInvalidInput).
			WithKind(richerror.KindInvalid).
			WithMeta(map[string]interface{}{"req": req}).
			WithErr(err)
	}

	return map[string]string{}, nil
}
